program roue_lorenz; (* La roue hydraulique de Lorenz *)

var 
   nbgod, n					     : integer;
   rayon,masse,fuitegod,source,frottement,pmaxgod,pi : real;
   ray,largeur,maxX,maxY,cx,cy			     : integer;
   
procedure initialise_parametres;
begin
   nbgod := 10;
   n := nbgod + 2;
   rayon := 0.5;
   masse := 2;
   source := 0.2;
   fuitegod := 0.02;
   pmaxgod := 0.3;
   frottement := 0.8;
   pi := 3.141592;
end; { initialise_parametres }

function sqr(x :  real): real;
begin
   sqr := x * x;
end;

procedure derive(    x : real; var y,yprime : array[1..20] of real);
var
   i,j			: integer;
   Minertie,Mforces,aux	: real;
begin
   for i := 1 to nbgod do
      if y[i] > 0 then yprime[i] := -fuitegod else yprime[i] := 0;
   aux := nbgod * y[nbgod+1]/pi + 1;
   j := trunc(aux/2) mod nbgod + 1;
   if y[j] < pmaxgod then yprime[j] := yprime[j] + source;
   yprime[nbgod+1] := y[nbgod+2];
   Minertie := masse;
   for i := 1 to nbgod do Minertie := Minertie + y[i];
   Minertie :=sqr(rayon) * Minertie;
   Mforces := 0;
   for i := 1 to nbgod do
      Mforces :=
        Mforces + y[i] * sin(y[nbgod+1] + 2*pi/nbgod*((nbgod+1-i) mod nbgod));
   Mforces := 9.81 * rayon * Mforces;
   yprime[nbgod+2] := (Mforces - frottement*y[nbgod+2])/Minertie;
end; { derive }

procedure RK4(	  x,h	: real; y : array[1..20] of real;
	      var nexty	: array[1..20] of real);
var
   k1,k2,k3,k4,tamp : array[1..20] of real;
   i		    : integer;
   h2		    : real;
begin 
   derive(x,y,k1);
   h2 := h/2;
   x := x + h2;
   for i := 1 to n do tamp[i] := y[i] + h2*k1[i];
   derive(x,tamp,k2);
   for i := 1 to n do tamp[i] := y[i] + h2*k2[i];
   derive(x,tamp,k3);
   x := x + h2;
   for i := 1 to n do tamp[i] := y[i] + h2*k3[i];
   derive(x,tamp,k4);
   for i := 1 to n do nexty[i] := y[i] + (h/6)*(k1[i]+2*k2[i]+2*k3[i]+k4[i]);
end; { RK4 }

procedure initgraphique;
begin
   maxX := 800;
   maxY := 600;
   open_screen(maxX,maxY);
   cx := maxX div 2;
   cy := maxY div 2;
   ray := 250;
   largeur := 15;
end; { initgraphique }


procedure trace_roue(var y : array[1..20] of real);
var
   scaly	 : real;
   px,py,ax,ay,i : integer;
   theta	 : real;
begin
   scaly := 1.0;
   clear_screen;
   theta := y[nbgod+1];
   ax := cx - trunc(ray * sin(theta + 2*pi/nbgod));
   ay := cy - trunc(scaly * ray * cos(theta + 2*pi/nbgod));
   for i := 1 to nbgod do begin
      px := cx - trunc(ray * sin(theta));
      py := cy - trunc(scaly * ray * cos(theta));
      fill_rect(px - largeur, py + 2*largeur - trunc(2*largeur*y[i]/pmaxgod),
		2 * largeur, trunc(2*largeur*y[i]/pmaxgod));
      draw_rect(px-largeur-1, py, 2*largeur, 2*largeur);
      draw_line(ax,ay,px,py);
      ax := px;
      ay := py;
      theta := theta - 2*pi/nbgod;
   end;
end; { trace_roue }

procedure simulation;
var
   i,cx,cy,k : integer;
   x,pas     : real;
   y	     : array[1..20] of real;
begin 
   x := 0;
   pas := 0.05;
   for i := 1 to nbgod do y[i] := 0;
   y[nbgod+1] := 0.1;
   y[nbgod+2] := 0;
   for k := 1 to 1000 do begin
      RK4(x,pas,y,y);
      x := x+pas;
      for i := 1 to nbgod do begin
	 if y[i] > pmaxgod then y[i] := pmaxgod;
	 if y[i] < 0 then y[i] := 0;
      end;
      while y[nbgod+1] >= 2*pi do y[nbgod+1] := y[nbgod+1] - 2*pi;
      while y[nbgod+1] < 0 do y[nbgod+1] := y[nbgod+1] + 2*pi;
      trace_roue(y);
      usleep(10000);
   end;
end; { simulation }

begin 
   initialise_parametres;
   initgraphique;
   simulation;
end.
